package sun.security.krb5.internal.crypto.dk;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.SecretKeyFactory;
import javax.crypto.Cipher;
import java.security.InvalidKeyException;
import javax.crypto.spec.DESKeySpec;
import java.security.GeneralSecurityException;
import java.util.Arrays;

public class Des3DkCrypto extends DkCrypto
{
    private static final byte[] ZERO_IV;
    
    public Des3DkCrypto() {
    }
    
    @Override
    protected int getKeySeedLength() {
        return 168;
    }
    
    public byte[] stringToKey(char[] array) throws GeneralSecurityException {
        byte[] charToUtf8 = null;
        try {
            charToUtf8 = DkCrypto.charToUtf8(array);
            byte[] stringToKey = this.stringToKey(charToUtf8, (byte[])null);
        }
        finally {
            if (charToUtf8 != null) {
                Arrays.fill(charToUtf8, (byte)0);
            }
        }
    }
    
    private byte[] stringToKey(byte[] array, byte[] array2) throws GeneralSecurityException {
        if (array2 != null && array2.length > 0) {
            throw new RuntimeException("Invalid parameter to stringToKey");
        }
        return this.dk(this.randomToKey(DkCrypto.nfold(array, this.getKeySeedLength())), Des3DkCrypto.KERBEROS_CONSTANT);
    }
    
    public byte[] parityFix(byte[] parityBit) throws GeneralSecurityException {
        setParityBit(parityBit);
        return parityBit;
    }
    
    @Override
    protected byte[] randomToKey(byte[] array) {
        if (array.length != 21) {
            throw new IllegalArgumentException("input must be 168 bits");
        }
        byte[] keyCorrection = keyCorrection(des3Expand(array, 0, 7));
        byte[] keyCorrection2 = keyCorrection(des3Expand(array, 7, 14));
        byte[] keyCorrection3 = keyCorrection(des3Expand(array, 14, 21));
        byte[] array2 = new byte[24];
        System.arraycopy(keyCorrection, 0, array2, 0, 8);
        System.arraycopy(keyCorrection2, 0, array2, 8, 8);
        System.arraycopy(keyCorrection3, 0, array2, 16, 8);
        return array2;
    }
    
    private static byte[] keyCorrection(byte[] key) {
        try {
            if (DESKeySpec.isWeak(key, 0)) {
                key[7] ^= 0xF0;
            }
        }
        catch (InvalidKeyException ex) {}
        return key;
    }
    
    private static byte[] des3Expand(byte[] array, int i, int j) {
        if (j - i != 7) {
            throw new IllegalArgumentException("Invalid length of DES Key Value:" + i + "," + j);
        }
        byte[] parityBit = new byte[8];
        int n = 0;
        System.arraycopy(array, i, parityBit, 0, 7);
        int n2 = 0;
        int n3;
        for (; i < j; ++i) {
            n3 = (byte)(array[i] & 0x1);
            n2 = (byte)(n2 + 1);
            if (n3 != 0) {
                n = (byte)(n | n3 << n2);
            }
        }
        parityBit[7] = n;
        setParityBit(parityBit);
        return parityBit;
    }
    
    private static void setParityBit(byte[] array) {
        for (int i = 0; i < array.length; ++i) {
            int j = array[i] & 0xFE;
            array[i] = (byte)(j | ((Integer.bitCount(j) & 0x1) ^ 0x1));
        }
    }
    
    @Override
    protected Cipher getCipher(byte[] key, byte[] zero_IV, int opmode) throws GeneralSecurityException {
        SecretKey generateSecret = SecretKeyFactory.getInstance("desede").generateSecret(new DESedeKeySpec(key, 0));
        if (zero_IV == null) {
            zero_IV = Des3DkCrypto.ZERO_IV;
        }
        Cipher instance = Cipher.getInstance("DESede/CBC/NoPadding");
        instance.init(opmode, generateSecret, new IvParameterSpec(zero_IV, 0, zero_IV.length));
        return instance;
    }
    
    @Override
    public int getChecksumLength() {
        return 20;
    }
    
    @Override
    protected byte[] getHmac(byte[] key, byte[] input) throws GeneralSecurityException {
        SecretKeySpec key2 = new SecretKeySpec(key, "HmacSHA1");
        Mac instance = Mac.getInstance("HmacSHA1");
        instance.init(key2);
        return instance.doFinal(input);
    }
    
    static {
        ZERO_IV = new byte[] { 0, 0, 0, 0, 0, 0, 0, 0 };
    }
}
